﻿using System;
//using System.Collections.Generic;
using System.Threading;
using Opto22.HART;

namespace Examples
{
    class Examples
    {
        const String targetIP = "10.192.57.180"; //Replace with your PAC brain/controller IP address
        const Int32 moduleNumber = 15;           //Replace with your module position on the rack
        const Int32 channelNumber = 0;           //Replace with the channel you are using

        static void Main(string[] args)
        {
            HART HartDemo = new HART(targetIP, moduleNumber, channelNumber);

            //A HART field device with polling address between 0 and 4 inclusive must be connected to the target module
            Console.WriteLine("Searching for HART field devices");
            byte[] pollingAddresses;
            byte[][] uniqueAddresses;
            HartDemo.FindDevices(0, 4, out pollingAddresses, out uniqueAddresses);
            for (int i = 0; i < pollingAddresses.Length; i++)
            {
                Console.WriteLine("Polling address: 0x{0}", pollingAddresses[i].ToString("X2"));
                HartDemo.PrintBytes(uniqueAddresses[i]);
            }
            Console.WriteLine();

            //If we found a device, continue with the demo
            if (pollingAddresses.Length > 0)
            {
                //Get found addresses to use for the rest of the demo
                byte[] pollAdd = new byte[] { pollingAddresses[0] };
                byte[] uniqueAdd = uniqueAddresses[0];
                int Result = 0;



                //Retrieve device data using the found polling address and Command 0
                //This gives an example of a blocking SendReceive
                Console.WriteLine("Blocking Command 0 Read");
                byte[] txData = new byte[0], rxHart, rxData;
                Result = HartDemo.SendReceive(pollAdd, 0, ref txData, out rxHart, out rxData, 5000);
                HartDemo.PrintBytes(rxHart);
                HartDemo.PrintBytes(rxData);
                Console.WriteLine();



                //Read the device's primary variable using the found unique address and Command 1
                //This shows an example of a non-blocking SendReceive
                Console.WriteLine("Non-blocking Command 1 Read");
                Result = 0;
                txData = new byte[0]; //Since txData is a ref, it contains the last message 
                                      //sent to the Opto 22 module and must be cleared before
                                      //sending a new message
                Result = HartDemo.SendReceive(uniqueAdd, 1, ref txData, out rxHart, out rxData, 0);
                Console.Write("Waiting for HART response");
                while (Result != 1)
                {
                    Result = HartDemo.Receive(out rxHart, out rxData, 0);
                    Console.Write(".");
                    Thread.Sleep(10);
                }
                Console.WriteLine();
                float primaryVariable = HartDemo.Unpackf32(rxHart, 3);
                Console.WriteLine("Unpacked primary variable: {0}", primaryVariable);
                Console.WriteLine();



                //Writes the devices Tag, Descriptor, and Date using Command 18
                //Demonstrates packing/unpacking Packed ASCII and Date formats
                Console.WriteLine("Pack/Unpack Packed ASCII and Date");
                Result = 0;
                txData = new byte[8];
                HartDemo.PackPAsc(ref txData, "OPTO 22", 0);
                HartDemo.PackDate(ref txData, 18, 3, 2014, 18);
                HartDemo.PackPAsc(ref txData, "OPTO22 HART DEMO", 6);
                Result = HartDemo.SendReceive(uniqueAdd, 18, ref txData, out rxHart, out rxData, 5000);
                string unpackedTag = HartDemo.UnpackPAsc(rxHart, 6, 2);
                string unpackedDescriptor = HartDemo.UnpackPAsc(rxHart, 12, 8);
                byte day, month;
                UInt16 year;
                HartDemo.UnpackDate(rxHart, 20, out day, out month, out year);
                Console.WriteLine("Unpacked Tag: {0}", unpackedTag);
                Console.WriteLine("Unpacked Descriptor: {0}", unpackedDescriptor);
                Console.WriteLine("Unpacked Date: {0}/{1}/{2}", day, month, year);

                Console.WriteLine();
            }
            else
            {
                Console.WriteLine("No HART device detected");
                Console.WriteLine("Please check your HART device connection to the Opto 22 module.");
            }


            Console.WriteLine("Press any key to exit");
            Console.Read();
        }
    }
}
